Package: reporter
Type: Package
Title: Creates Statistical Reports
Version: 1.4.7
Authors@R: c( 
    person("David", "Bosak", email = "dbosak01@gmail.com", role = c("aut", "cre")),
    person("Bill", "Huang", email = "Bill.Huang@Toastmasters.org.tw", role = c("aut")),
    person("Kevin", "Kramer", email = "kkrame02@amgen.com", role = "ctb"),
    person("Duong", "Tran", email = "trand000@aol.com", role = "ctb"),
    person("Raphael", "Huang", email = "chi-hua.huang@astellas.com", role = "ctb"),
    person("Archytas Clinical Solutions", role = "cph")
    )
Maintainer: David Bosak <dbosak01@gmail.com>
Description: Contains functions to create regulatory-style statistical reports.
    Originally designed to create tables, listings, and figures for the 
    pharmaceutical, biotechnology, and medical device industries, these
    reports are generalized enough that they could be used in any industry.
    Generates text, rich-text, PDF, HTML, and Microsoft Word file formats.  
    The package specializes 
    in printing wide and long tables with automatic page wrapping and splitting.  
    Reports can be produced with a minimum of function calls, and without 
    relying on other table packages.  The package supports titles, footnotes, 
    page header, page footers, spanning headers, page by variables, 
    and automatic page numbering.
License: CC0
Encoding: UTF-8
URL: https://reporter.r-sassy.org, https://github.com/dbosak01/reporter
BugReports: https://github.com/dbosak01/reporter/issues
Depends: R (>= 3.6), common (>= 1.1.0)
Suggests: testthat, magrittr, dplyr, tidyr, readr, knitr, rmarkdown,
        ggplot2, gridExtra, survminer, utils, logr, covr, libr, devEMF,
        patchwork, tools
Imports: fmtr(>= 1.5.8), stringi, crayon, jpeg, graphics, grDevices,
        zip, withr, glue
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-15 04:21:35 UTC; dbosa
Author: David Bosak [aut, cre],
  Bill Huang [aut],
  Kevin Kramer [ctb],
  Duong Tran [ctb],
  Raphael Huang [ctb],
  Archytas Clinical Solutions [cph]
Repository: CRAN
Date/Publication: 2026-05-15 04:50:02 UTC
Built: R 4.6.0; ; 2026-05-15 06:00:10 UTC; unix
