# Generated by roxygen2: do not edit by hand

S3method(print,plot_spec)
S3method(print,report_spec)
S3method(print,style_spec)
S3method(print,table_spec)
S3method(print,text_spec)
S3method(render,default)
S3method(render,pdf_array)
S3method(render,pdf_dictionary)
S3method(render,pdf_document)
S3method(render,pdf_image_stream)
S3method(render,pdf_info)
S3method(render,pdf_object)
S3method(render,pdf_text_stream)
export(add_content)
export(add_style)
export(cell_style)
export(column_defaults)
export(create_plot)
export(create_report)
export(create_style)
export(create_table)
export(create_text)
export(define)
export(footer_image)
export(footnotes)
export(ftn)
export(get_theme)
export(header_image)
export(lowcase_n)
export(lowcase_parens)
export(options_fixed)
export(page_by)
export(page_footer)
export(page_header)
export(set_margins)
export(span)
export(spanning_header)
export(stub)
export(title_header)
export(titles)
export(ttl)
export(upcase_n)
export(upcase_parens)
export(write_registration_file)
export(write_report)
import(common)
import(crayon)
import(fmtr)
import(glue)
import(grDevices)
import(graphics)
import(jpeg)
import(stats)
import(stringi)
import(withr)
import(zip)
